<?php
if ( !defined( 'DB_NAME' ) ) {
	header( 'HTTP/1.0 403 Forbidden' );
	die;
}

/**
 * Name: Auto Remove Empty Meta Value
 * Mô tả: Lọc ra những thẻ meta trống khi đăng bài và xóa nó trước khi nó được 
 * lưu vào database bởi nó hoàn toàn vô dụng và gây lãng phí tài nguyên.
 */

add_action('save_post','bbit_auto_remove_meta');
function bbit_auto_remove_meta($post_id) { 

    if ( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) return;
    if (!current_user_can('edit_post', $post_id)) return;
    $custom_fields = get_post_custom($post_id);
    if(!$custom_fields) return;

    foreach($custom_fields as $key=>$custom_field) {

        $values = array_filter($custom_field);

        if(empty($values)):
            delete_post_meta($post_id, $key);
        endif;
		
    }
    return;
}

/**
 * Name: Auto Add Tag Meta Value
 * Mô tả: Tự động thêm Tag nếu tên nhà phát hành/tác giả/ca sĩ...được điền
 * Nếu giá trị tồn tại sẽ không làm gì cả
 */

add_action('save_post', 'bbit_auto_tag');
function bbit_auto_tag(){

    $post_id = get_the_ID();
	
    if ( defined('DOING_AUTOSAVE') && DOING_AUTOSAVE ) return;
    if (!current_user_can('edit_post', $post_id)) return;

	$value = get_post_meta($post_id, 'bbit_phathanh', true);
	$value2 = get_post_meta($post_id, 'bbit_author2', true);		
	$value3 = get_post_meta($post_id, 'bbit_author', true);
	$value4 = get_post_meta($post_id, 'bbit_singer', true);	
    $tags = get_tags( array('hide_empty' => false) );

	foreach ($tags as $tag) {
		if( $tag->name != $value ) {
			wp_set_post_tags( $post_id, $value, true);
		}
			
		if($tag->name != $value2) {
			wp_set_post_tags( $post_id, $value2, true);
		}
			
		if($tag->name != $value3) {
			wp_set_post_tags( $post_id, $value3, true);
		}
		
		if($tag->name != $value4) {
			wp_set_post_tags( $post_id, $value4, true);
		}		
    }
	
	return;
}
 
/**
 * Name: Auto Remove Query String
 * Mô tả: Tự động gỡ bỏ Query String có dạng ?ver= trong url của js và css
 * có lợi cho hiệu suất và tốc độ tải của website
 */

function _remove_query_strings_1( $src ){
	$rqs = explode( '?ver', $src );
        return $rqs[0];
}
function _remove_query_strings_2( $src ){
	$rqs = explode( '&ver', $src );
        return $rqs[0];
}
add_filter( 'script_loader_src', '_remove_query_strings_1', 15, 1 );
add_filter( 'style_loader_src', '_remove_query_strings_1', 15, 1 );
add_filter( 'script_loader_src', '_remove_query_strings_2', 15, 1 );
add_filter( 'style_loader_src', '_remove_query_strings_2', 15, 1 );

/**
 * Name: Clean up the <head>
 * Mô tả: Làm sạch wp_head khỏi những thẻ không cần thiết, tối ưu cho tốc độ  
 */

/**
 * Name: Module Not Outlink
 * Mô tả: Tự động thêm rel nofollow vào những liên kết ra ngoài để báo cho bot
 * không theo liên kết đó giúp hạn chế nguy cơ giảm Pagerank.
 */
add_filter( 'the_content', 'bbit_nofollow');
function bbit_nofollow( $content ) {

	$regexp = "<a\s[^>]*href=(\"??)([^\" >]*?)\\1[^>]*>";
	
	if(preg_match_all("/$regexp/siU", $content, $matches, PREG_SET_ORDER)) {
		if( !empty($matches) ) {
			$srcUrl = get_option('siteurl');
			for ($i=0; $i < count($matches); $i++)
			{
				$tag = $matches[$i][0];
				$tag2 = $matches[$i][0];
				$url = $matches[$i][0];
				$noFollow = '';
				$pattern = '/rel\s*=\s*"\s*[n|d]ofollow\s*"/';
				preg_match($pattern, $tag2, $match, PREG_OFFSET_CAPTURE);
				if( count($match) < 1 )
					$noFollow .= ' rel="nofollow" ';
				$pos = strpos($url,$srcUrl);
				if ($pos === false) {
					$tag = rtrim ($tag,'>');
					$tag .= $noFollow.'>';
					$content = str_replace($tag2,$tag,$content);
				}
			}
		}
	}
	
	$content = str_replace(']]>', ']]&gt;', $content);
	return $content;
}

/**
 * Name: Image SEO
 * Mô tả: Tự động tạo ra ALT và Title cho hình ảnh trong bài
 */
function seo_friendly_images_process($matches) {
	global $post;
	$override= 'on';
	$override_title= 'on';
	$matches[0]=preg_replace('|([\'"])[/ ]*$|', '\1 /', $matches[0]);
	$matches[0] = preg_replace('/\s*=\s*/', '=', substr($matches[0],0,strlen($matches[0])-2));
	preg_match('/src\s*=\s*([\'"])?((?(1).+?|[^\s>]+))(?(1)\1)/', $matches[0], $source);
	$saved=$source[2];
	preg_match('%[^/]+(?=\.[a-z]{3}\z)%', $source[2], $source);
	$pieces = preg_split('/(\w+=)/', $matches[0], -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
	$image_name = str_replace("-", " ", $source[0]);
	if (!in_array('title=', $pieces) || $override_title=="on") {
		if (!in_array('title=', $pieces)) {
			array_push($pieces, ' title="Hình ảnh '.$image_name.' in '.$post->post_title.'"');
		} else {
			$key=array_search('title=',$pieces);
			$pieces[$key+1]='"Hình ảnh '.$image_name.' in '.$post->post_title.'"';
		}
	}
	
	if (!in_array('alt=', $pieces) || $override=="on" ) {
		
		if (!in_array('alt=', $pieces)) {
			array_push($pieces, ' alt="Hình ảnh '.$image_name.' in '.$post->post_title.'"');
		} else {
			$key=array_search('alt=',$pieces);
			$pieces[$key+1]='"Hình ảnh '.$image_name.' in '.$post->post_title.'" ';
		}
	}
	return implode('', $pieces).' /';
}

function seo_friendly_images($content) {
	return preg_replace_callback('/<img[^>]+/', 'seo_friendly_images_process', $content);
}
add_filter('the_content', 'seo_friendly_images', 100);

add_shortcode('wp_caption', 'fixed_img_caption_shortcode');
add_shortcode('caption', 'fixed_img_caption_shortcode');
function fixed_img_caption_shortcode($attr, $content = null) {
	if ( ! isset( $attr['caption'] ) ) {
		if ( preg_match( '#((?:<a [^>]+>\s*)?<img [^>]+>(?:\s*</a>)?)(.*)#is', $content, $matches ) ) {
			$content = $matches[1];
			$attr['caption'] = trim( $matches[2] );
		}
	}
	$output = apply_filters('img_caption_shortcode', '', $attr, $content);
	if ( $output != '' )
		return $output;
	extract(shortcode_atts(array(
		'id'	=> '',
		'align'	=> '',
		'width'	=> '',
		'caption' => ''
	), $attr));
	if ( 1 > (int) $width || empty($caption) )
		return $content;
	if ( $id ) $id = 'id="' . esc_attr($id) . '" ';
	return '<div class="wp-caption">'
	. do_shortcode( $content ) . '<p class="wp-caption-text">' . $caption . '</p></div>';
}