<?php

if ( ! defined( 'WPSEO_VERSION' ) ) {
	header( 'HTTP/1.0 403 Forbidden' );
	die;
}

class WPSEO_Metabox {

	var $meta_length = 156;

	var $meta_length_reason = '';

	function __construct() {
		if ( ! class_exists( 'Yoast_TextStatistics' ) && apply_filters( 'wpseo_use_page_analysis', true ) === true )
			require_once( WPSEO_PATH . 'admin/TextStatistics.php' );

		add_action( 'add_meta_boxes', array( $this, 'add_meta_box' ) );
		add_action( 'admin_print_styles-post-new.php', array( $this, 'enqueue' ) );
		add_action( 'admin_print_styles-post.php', array( $this, 'enqueue' ) );
		add_action( 'admin_print_styles-edit.php', array( $this, 'enqueue' ) );
		add_action( 'admin_head', array( $this, 'script' ) );
		add_action( 'wp_insert_post', array( $this, 'save_postdata' ) );
		add_action( 'edit_attachment', array( $this, 'save_postdata' ) );
		add_action( 'add_attachment', array( $this, 'save_postdata' ) );
		add_action( 'admin_init', array( $this, 'setup_page_analysis' ) );
	}

	public function setup_page_analysis() {

		if ( apply_filters( 'wpseo_use_page_analysis', true ) === true ) {

			$options = get_wpseo_options();

			foreach ( get_post_types( array( 'public' => true ), 'names' ) as $pt ) {
				if ( isset( $options['hideeditbox-' . $pt] ) && $options['hideeditbox-' . $pt] )
					continue;
			}

			add_action( 'restrict_manage_posts', array( $this, 'posts_filter_dropdown' ) );
		}

	}

	public function strtolower_utf8( $string ) {
		
		// Prevent comparison between utf8 characters and html entities (é vs &eacute;)
		$string = html_entity_decode( $string );
		
		$convert_to  = array(
        "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "Â", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a", "a",
        "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e", "e",
        "i", "i", "i", "i", "i", "i", "i", "i", "i", "i", "i",
        "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o", "o",
        "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u", "u",
        "y", "y", "y", "y", "y", "y", "y", "y", "y", "y", "y",
        "d", "d", "d",
        "q", "w", "r", "t", "p", "s", "f", "g", "h", "j", "k", "l", "z", "x", "c", "v", "b", "n", "m"
        );
        $convert_from = array(
        "à", "á", "ạ", "ả", "ã", "â", "ầ", "ấ", "ậ", "ẩ", "ẫ", "ă", "ằ", "ắ", "ặ", "ẳ", "ẵ", "A", "À", "Á", "Ạ", "Ả", "Ã", "Â", "Ầ", "Ấ", "Ậ", "Ẩ", "Ẫ", "Ă", "Ằ", "Ắ", "Ặ", "Ẳ", "Ẵ",
        "è", "é", "ẹ", "ẻ", "ê", "ề", "ế", "ệ", "ể", "ễ", "E", "È", "É", "Ẹ", "Ẻ", "Ẽ", "Ê", "Ề", "Ế", "Ệ", "Ể", "Ễ",
        "ì", "í", "ị", "ỉ", "ĩ", "I", "Ì", "Í", "Ị", "Ỉ", "Ĩ",
        "ò", "ó", "ọ", "ỏ", "õ", "ô", "ồ", "ố", "ộ", "ổ", "ỗ", "ơ", "ờ", "ớ", "ợ", "ở", "ỡ", "O", "Ò", "Ó", "Ọ", "Ỏ", "Õ", "Ô", "Ồ", "Ố", "Ộ", "Ổ", "Ỗ", "Ơ", "Ờ", "Ớ", "Ợ", "Ở", "Ỡ",
        "ù", "ú", "ụ", "ủ", "ũ", "ư", "ừ", "ứ", "ự", "ử", "ữ", "U", "Ù", "Ú", "Ụ", "Ủ", "Ũ", "Ư", "Ừ", "Ứ", "Ự", "Ử", "Ữ",
        "ỳ", "ý", "ỵ", "ỷ", "ỹ", "Y", "Ỳ", "Ý", "Ỵ", "Ỷ", "Ỹ",
        "đ", "D", "Đ",
        "Q", "W", "R", "T", "P", "S", "F", "G", "H", "J", "K", "L", "Z", "X", "C", "V", "B", "N", "M"
        );

		return str_replace( $convert_from, $convert_to, $string );
	}

	public function add_meta_box() {
		$options = get_wpseo_options();

		foreach ( get_post_types( array( 'public' => true ) ) as $posttype ) {
			if ( isset( $options['hideeditbox-' . $posttype] ) && $options['hideeditbox-' . $posttype] )
				continue;
			add_meta_box( 'wpseo_meta', __( 'SEO', 'wordpress-seo' ), array( $this, 'meta_box' ), $posttype, 'normal', apply_filters( 'wpseo_metabox_prio', 'high' ) );
		}
	}

	public function add_custom_box() {
		_deprecated_function( __FUNCTION__, 'WPSEO 1.5.0', 'WPSEO_Metabox::add_meta_box()' );
		$this->add_meta_box();
	}

	public function script() {
		if ( isset( $_GET['post'] ) ) {
			$post_id = (int) $_GET['post'];
			$post    = get_post( $post_id );
		}
		else {
			global $post;
		}

		if ( ! isset( $post ) )
			return;

		$options = get_wpseo_options();

		$date = '';
		if ( isset( $options['showdate-' . $post->post_type] ) && $options['showdate-' . $post->post_type] ) {
			$date = $this->get_post_date( $post );

			$this->meta_length        = $this->meta_length - ( strlen( $date ) + 5 );
			$this->meta_length_reason = __( ' (because of date display)', 'wordpress-seo' );
		}

		$this->meta_length_reason = apply_filters( 'wpseo_metadesc_length_reason', $this->meta_length_reason, $post );
		$this->meta_length        = apply_filters( 'wpseo_metadesc_length', $this->meta_length, $post );

		unset( $date );

		$title_template = '';
		if ( isset( $options['title-' . $post->post_type] ) )
			$title_template = $options['title-' . $post->post_type];

		// If there's no title template set, use the default, otherwise title preview won't work.
		if ( $title_template == '' )
			$title_template = '%%title%% - %%sitename%%';
		$title_template = wpseo_replace_vars( $title_template, $post, array( '%%title%%' ) );

		$metadesc_template = '';
		if ( isset( $options['metadesc-' . $post->post_type] ) )
			$metadesc_template = wpseo_replace_vars( $options['metadesc-' . $post->post_type], $post, array( '%%excerpt%%', '%%excerpt_only%%' ) );

		$sample_permalink = get_sample_permalink( $post->ID );
		$sample_permalink = str_replace( '%page', '%post', $sample_permalink[0] );
		?>
		<script type="text/javascript">
			var wpseo_lang = '<?php echo substr( get_locale(), 0, 2 ); ?>';
			var wpseo_meta_desc_length = '<?php echo $this->meta_length; ?>';
			var wpseo_title_template = '<?php echo esc_attr( $title_template ); ?>';
			var wpseo_metadesc_template = '<?php echo esc_attr( $metadesc_template ); ?>';
			var wpseo_permalink_template = '<?php echo esc_url( $sample_permalink ); ?>';
			var wpseo_keyword_suggest_nonce = '<?php echo wp_create_nonce( 'wpseo-get-suggest' ); ?>';
		</script>
	<?php
	}

	public function do_tab( $id, $heading, $content ) {
		?>
		<div class="wpseotab <?php echo esc_attr( $id ) ?>">
			<h4 class="wpseo-heading"><?php echo esc_html( $heading ); ?></h4>
			<table class="form-table">
				<?php echo $content ?>
			</table>
		</div>
	<?php
	}

	public function get_meta_boxes( $post_type = 'post' ) {
		$options = get_wpseo_options();

		$mbs                   = array();
		$mbs['snippetpreview'] = array(
			"name"  => "snippetpreview",
			"type"  => "snippetpreview",
			"title" => __( "Snippet Preview", 'wordpress-seo' ),
		);

		$mbs['title']          = array(
			"name"        => "title",
			"std"         => "",
			"type"        => "text",
			"title"       => __( "SEO Title", 'wordpress-seo' ),
			"description" => sprintf( __( "Tiêu đề hiển thị trên công cụ tìm kiếm, tối đa 56 kí tự, còn lại %s kí tự.", 'wordpress-seo' ), "<span id='yoast_wpseo_title-length'></span>" )
		);
		$mbs['metadesc']       = array(
			"name"        => "metadesc",
			"std"         => "",
			"class"       => "metadesc",
			"type"        => "textarea",
			"title"       => __( "Meta Description", 'wordpress-seo' ),
			"rows"        => 2,
			"richedit"    => false,
			"description" => sprintf( __( "Thẻ <code>meta</code> description mô tả về bài viết, nên chứa từ khóa. Tối đa %s kí tự%s, còn lại %s kí tự.", 'wordpress-seo' ), $this->meta_length, $this->meta_length_reason, "<span id='yoast_wpseo_metadesc-length'></span>" ) . " <div id='yoast_wpseo_metadesc_notice'></div>",
		);
		if ( isset( $options['usemetakeywords'] ) && $options['usemetakeywords'] ) {
			$mbs['metakeywords'] = array(
				"name"        => "metakeywords",
				"std"         => "",
				"class"       => "metakeywords",
				"type"        => "text",
				"title"       => __( "Meta Keywords", 'wordpress-seo' ),
				"description" => sprintf( __( "If you type something above it will override your %smeta keywords template%s.", 'wordpress-seo' ), "<a target='_blank' href='" . admin_url( 'admin.php?page=wpseo_titles#' . esc_url( $post_type ) ) . "'>", "</a>" )
			);
		}

		// Apply filters before entering the advanced section
		$mbs = apply_filters( 'wpseo_metabox_entries', $mbs );

		return $mbs;
	}

	function get_advanced_meta_boxes() {
		global $post;

		$post_type = '';
		if ( isset( $post->post_type ) )
			$post_type = $post->post_type;
		else if ( ! isset( $post->post_type ) && isset( $_GET['post_type'] ) )
			$post_type = $_GET['post_type'];

		$options = get_wpseo_options();

		$mbs = array();

		$mbs['meta-robots-noindex']  = array(
			"name"    => "meta-robots-noindex",
			"std"     => "-",
			"title"   => __( "Meta Robots Index", 'wordpress-seo' ),
			"type"    => "select",
			"options" => array(
				"0" => sprintf( __( "Default for post type, currently: %s", 'wordpress-seo' ), ( isset( $options['noindex-' . $post_type] ) && $options['noindex-' . $post_type] ) ? 'noindex' : 'index' ),
				"2" => __( "index", 'wordpress-seo' ),
				"1" => __( "noindex", 'wordpress-seo' ),
			),
		);
		$mbs['meta-robots-nofollow'] = array(
			"name"    => "meta-robots-nofollow",
			"std"     => "follow",
			"title"   => __( "Meta Robots Follow", 'wordpress-seo' ),
			"type"    => "radio",
			"options" => array(
				"0" => __( "Follow", 'wordpress-seo' ),
				"1" => __( "Nofollow", 'wordpress-seo' ),
			),
		);
		$mbs['meta-robots-adv']      = array(
			"name"        => "meta-robots-adv",
			"std"         => "none",
			"type"        => "multiselect",
			"title"       => __( "Meta Robots Advanced", 'wordpress-seo' ),
			"description" => __( "Advanced <code>meta</code> robots settings for this page.", 'wordpress-seo' ),
			"options"     => array(
				"noodp"     => __( "NO ODP", 'wordpress-seo' ),
				"noydir"    => __( "NO YDIR", 'wordpress-seo' ),
				"noarchive" => __( "No Archive", 'wordpress-seo' ),
				"nosnippet" => __( "No Snippet", 'wordpress-seo' ),
			),
		);
		if ( isset( $options['enablexmlsitemap'] ) && $options['enablexmlsitemap'] ) {
			$mbs['sitemap-include'] = array(
				"name"        => "sitemap-include",
				"std"         => "-",
				"type"        => "select",
				"title"       => __( "Include in Sitemap", 'wordpress-seo' ),
				"description" => __( "Should this page be in the XML Sitemap at all times, regardless of Robots Meta settings?", 'wordpress-seo' ),
				"options"     => array(
					"-"      => __( "Auto detect", 'wordpress-seo' ),
					"always" => __( "Always include", 'wordpress-seo' ),
					"never"  => __( "Never include", 'wordpress-seo' ),
				),
			);
			$mbs['sitemap-prio']    = array(
				"name"        => "sitemap-prio",
				"std"         => "-",
				"type"        => "select",
				"title"       => __( "Sitemap Priority", 'wordpress-seo' ),
				"description" => __( "The priority given to this page in the XML sitemap.", 'wordpress-seo' ),
				"options"     => array(
					"-"   => __( "Automatic prioritization", 'wordpress-seo' ),
					"1"   => __( "1 - Highest priority", 'wordpress-seo' ),
					"0.9" => "0.9",
					"0.8" => "0.8 - " . __( "Default for first tier pages", 'wordpress-seo' ),
					"0.7" => "0.7",
					"0.6" => "0.6 - " . __( "Default for second tier pages and posts", 'wordpress-seo' ),
					"0.5" => "0.5 - " . __( "Medium priority", 'wordpress-seo' ),
					"0.4" => "0.4",
					"0.3" => "0.3",
					"0.2" => "0.2",
					"0.1" => "0.1 - " . __( "Lowest priority", 'wordpress-seo' ),
				),
			);
		}
		$mbs['sitemap-html-include'] = array(
			"name"        => "sitemap-html-include",
			"std"         => "-",
			"type"        => "select",
			"title"       => __( "Include in HTML Sitemap", 'wordpress-seo' ),
			"description" => __( "Should this page be in the HTML Sitemap at all times, regardless of Robots Meta settings?", 'wordpress-seo' ),
			"options"     => array(
				"-"      => __( "Auto detect", 'wordpress-seo' ),
				"always" => __( "Always include", 'wordpress-seo' ),
				"never"  => __( "Never include", 'wordpress-seo' ),
			),
		);
		$mbs['canonical']            = array(
			"name"        => "canonical",
			"std"         => "",
			"type"        => "text",
			"title"       => __( "Canonical URL", 'wordpress-seo' ),
			"description" => sprintf( __( "The canonical URL that this page should point to, leave empty to default to permalink. %sCross domain canonical%s supported too.", 'wordpress-seo' ), "<a target='_blank' href='http://googlewebmastercentral.blogspot.com/2009/12/handling-legitimate-cross-domain.html'>", "</a>" )
		);
		$mbs['redirect']             = array(
			"name"        => "redirect",
			"std"         => "",
			"type"        => "text",
			"title"       => __( "301 Redirect", 'wordpress-seo' ),
			"description" => __( "The URL that this page should redirect to.", 'wordpress-seo' )
		);

		// Apply filters for in advanced section
		$mbs = apply_filters( 'wpseo_metabox_entries_advanced', $mbs );

		return $mbs;
	}

	/**
	 * Output the meta box
	 */
	function meta_box() {
		if ( isset( $_GET['post'] ) ) {
			$post_id = (int) $_GET['post'];
			$post    = get_post( $post_id );
		}
		else {
			global $post;
		}

		$options = get_wpseo_options();

		?>
		<div class="wpseo-metabox-tabs-div">
		<ul class="wpseo-metabox-tabs" id="wpseo-metabox-tabs">
			<li class="general"><a class="wpseo_tablink" href="#wpseo_general">SEO cơ bản</a></li>
			<?php if ( current_user_can( 'manage_options' ) || ! isset( $options['disableadvanced_meta'] ) || ! $options['disableadvanced_meta'] ): ?>
			<li class="advanced"><a class="wpseo_tablink" href="#wpseo_advanced"><?php _e( "Advanced", 'wordpress-seo' ); ?></a></li>
			<?php endif; ?>
			<?php do_action( 'wpseo_tab_header' ); ?>
		</ul>
		<?php
		$content = '';
		foreach ( $this->get_meta_boxes( $post->post_type ) as $meta_box ) {
			$content .= $this->do_meta_box( $meta_box );
		}
		$this->do_tab( 'general', __( 'SEO cơ bản', 'wordpress-seo' ), $content );

		if ( current_user_can( 'manage_options' ) || ! isset( $options['disableadvanced_meta'] ) || ! $options['disableadvanced_meta'] ) {
			$content = '';
			foreach ( $this->get_advanced_meta_boxes() as $meta_box ) {
				$content .= $this->do_meta_box( $meta_box );
			}
			$this->do_tab( 'advanced', __( 'Advanced', 'wordpress-seo' ), $content );
		}

		do_action( 'wpseo_tab_content' );

		echo '</div>';
	}

	function do_meta_box( $meta_box ) {
		$content        = '';
		$meta_box_value = '';

		if ( ! isset( $meta_box['name'] ) ) {
			$meta_box['name'] = '';
		}
		else {
			if ( wpseo_get_value( $meta_box['name'] ) !== false ) {
				$meta_box_value = wpseo_get_value( $meta_box['name'] );
			}
			else if ( isset( $meta_box['std'] ) ) {
				$meta_box_value = $meta_box['std'];
			}
			$meta_box['name'] = esc_attr( $meta_box['name'] );
		}

		$class = '';
		if ( ! empty( $meta_box['class'] ) )
			$class = ' ' . $meta_box['class'];

		$placeholder = '';
		if ( isset( $meta_box['placeholder'] ) && ! empty( $meta_box['placeholder'] ) )
			$placeholder = $meta_box['placeholder'];

		$help = '';
		if ( isset( $meta_box['help'] ) && $meta_box['help'] )
			$help = '<img src="' . get_template_directory_uri().'images/question-mark.png'. '" class="alignright yoast_help" id="' . $meta_box['name'] . 'help" alt="' . esc_attr( $meta_box['help'] ) . '" />';

		$content .= '<tr>';
		$content .= '<th scope="row"><label for="yoast_wpseo_' . $meta_box['name'] . '">' . $meta_box['title'] . ':</label>' . $help . '</th>';
		$content .= '<td>';

		switch ( $meta_box['type'] ) {
			case "snippetpreview":
				$content .= $this->snippet();
				break;
			case "text":
				$ac = '';
				if ( isset( $meta_box['autocomplete'] ) && $meta_box['autocomplete'] == 'off' )
					$ac = 'autocomplete="off" ';
				$content .= '<input type="text" placeholder="' . esc_attr( $placeholder ) . '" id="yoast_wpseo_' . $meta_box['name'] . '" ' . $ac . 'name="yoast_wpseo_' . $meta_box['name'] . '" value="' . esc_attr( $meta_box_value ) . '" class="large-text"/><br />';
				break;
			case "textarea":
				$content .= '<textarea class="large-text" rows="3" id="yoast_wpseo_' . $meta_box['name'] . '" name="yoast_wpseo_' . $meta_box['name'] . '">' . esc_textarea( $meta_box_value ) . '</textarea>';
				break;
			case "select":
				$content .= '<select name="yoast_wpseo_' . $meta_box['name'] . '" id="yoast_wpseo_' . $meta_box['name'] . '" class="yoast' . $class . '">';
				foreach ( $meta_box['options'] as $val => $option ) {
					$selected = '';
					if ( $meta_box_value == $val )
						$selected = 'selected="selected"';
					$content .= '<option ' . $selected . ' value="' . esc_attr( $val ) . '">' . esc_html( $option ) . '</option>';
				}
				$content .= '</select>';
				break;
			case "multiselect":
				$selectedarr         = explode( ',', $meta_box_value );
				$meta_box['options'] = array( 'none' => 'None' ) + $meta_box['options'];
				$content .= '<select multiple="multiple" size="' . count( $meta_box['options'] ) . '" style="height: ' . ( count( $meta_box['options'] ) * 16 ) . 'px;" name="yoast_wpseo_' . $meta_box['name'] . '[]" id="yoast_wpseo_' . $meta_box['name'] . '" class="yoast' . $class . '">';
				foreach ( $meta_box['options'] as $val => $option ) {
					$selected = '';
					if ( in_array( $val, $selectedarr ) )
						$selected = 'selected="selected"';
					$content .= '<option ' . $selected . ' value="' . esc_attr( $val ) . '">' . esc_html( $option ) . '</option>';
				}
				$content .= '</select>';
				break;
			case "checkbox":
				$checked = '';
				if ( $meta_box_value == 'on' || $meta_box_value == true )
					$checked = 'checked="checked"';
				$expl = ( isset( $meta_box['expl'] ) ) ? esc_html( $meta_box['expl'] ) : '';
				$content .= '<input type="checkbox" id="yoast_wpseo_' . $meta_box['name'] . '" name="yoast_wpseo_' . $meta_box['name'] . '" ' . $checked . ' class="yoast' . $class . '"/> ' . $expl . '<br />';
				break;
			case "radio":
				if ( $meta_box_value == '' )
					$meta_box_value = $meta_box['std'];
				foreach ( $meta_box['options'] as $val => $option ) {
					$selected = '';
					if ( $meta_box_value == $val )
						$selected = 'checked="checked"';
					$content .= '<input type="radio" ' . $selected . ' id="yoast_wpseo_' . $meta_box['name'] . '_' . esc_attr( $val ) . '" name="yoast_wpseo_' . $meta_box['name'] . '" value="' . esc_attr( $val ) . '"/> <label for="yoast_wpseo_' . $meta_box['name'] . '_' . $val . '">' . $option . '</label> ';
				}
				break;
			case "upload":
				if ( $meta_box_value == '' )
					$meta_box_value = $meta_box['std'];
				$content .= '<label for="upload_image">';
				$content .= '<input id="yoast_wpseo_'.$meta_box['name'].'" type="text" size="36" name="yoast_wpseo_'.$meta_box['name'].'" value="' . $meta_box_value . '" />';
				$content .= '<input id="yoast_wpseo_'.$meta_box['name'].'_button" class="wpseo_image_upload_button button" type="button" value="Upload Image" />';
				$content .= '</label>';
				break;
			case "divtext":
				$content .= '<p>' . $meta_box['description'] . '</p>';
		}

		if ( isset( $meta_box['description'] ) )
			$content .= '<p>' . $meta_box['description'] . '</p>';

		$content .= '</td>';
		$content .= '</tr>';

		return $content;
	}

	function get_post_date( $post ) {
		if ( isset( $post->post_date ) && $post->post_status == 'publish' )
			$date = date( 'j M Y', strtotime( $post->post_date ) );
		else
			$date = date( 'j M Y' );
		return $date;
	}

	function snippet() {
		if ( isset( $_GET['post'] ) ) {
			$post_id = (int) $_GET['post'];
			$post    = get_post( $post_id );
		}
		else {
			global $post;
		}

		$options = get_wpseo_options();

		$date = '';
		if ( isset( $options['showdate-' . $post->post_type] ) && $options['showdate-' . $post->post_type] )
			$date = $this->get_post_date( $post );

		$title = wpseo_get_value( 'title' );
		$desc  = wpseo_get_value( 'metadesc' );

		$slug = $post->post_name;
		if ( empty( $slug ) )
			$slug = sanitize_title( $title );

		if ( ! empty( $date ) )
			$datestr = '<span style="color: #666;">' . $date . '</span> – ';
		else
			$datestr = '';
		$content = '<div id="wpseosnippet">
			<a class="title" href="#">' . esc_html( $title ) . '</a><br/>';

		$content .= '<a href="#" style="font-size: 13px; color: #282; line-height: 15px;" class="url">' . str_replace( 'http://', '', get_bloginfo( 'url' ) ) . '/' . esc_html( $slug ) . '/</a>';

		$content .= '<p class="desc" style="font-size: 13px; color: #000; line-height: 15px;">' . $datestr . '<span class="content">' . esc_html( $desc ) . '</span></p>';

		$content .= '</div>';

		$content = apply_filters( 'wpseo_snippet', $content, $post, compact( 'title', 'desc', 'date', 'slug' ) );

		return $content;
	}

	function get_gplus_data( $user_id ) {
		if ( $gplus = get_transient( 'gplus_' . $user_id ) )
			return $gplus;

		$gplus_profile = get_the_author_meta( 'googleplus', $user_id );

		if ( empty( $gplus_profile ) )
			return false;
		if ( preg_match( '`u/0/([^/]+)/`', $gplus_profile, $match ) )
			$gplus_id = $match[1];
		else if ( preg_match( '`\.com/(\d+)`', $gplus_profile, $match ) )
			$gplus_id = $match[1];
		else
			return false;

		$args = array(
			'headers' => array(
				'Referer' => 'http://yoast.com/wp-admin/',
			),
		);

		$resp = wp_remote_get( 'https://www.googleapis.com/plus/v1/people/' . $gplus_id . '?key=AIzaSyBLYmCW10gzW63ob8NYIPTneph1arsxqWs', $args );
		if ( ! is_wp_error( $resp ) ) {
			$gplus = json_decode( $resp['body'] );

			set_transient( 'gplus_' . $user_id, $gplus, ( 7 * 24 * 60 * 60 ) );

			return $gplus;
		}
		else {
			return false;
		}

	}

	function save_postdata( $post_id ) {

		if ( $post_id == null )
			return false;

		if ( wp_is_post_revision( $post_id ) )
			$post_id = wp_is_post_revision( $post_id );

		clean_post_cache( $post_id );
		$post = get_post( $post_id );

		$metaboxes = array_merge( $this->get_meta_boxes( $post->post_type ), $this->get_advanced_meta_boxes() );

		$metaboxes = apply_filters( 'wpseo_save_metaboxes', $metaboxes );

		foreach ( $metaboxes as $meta_box ) {
			if ( ! isset( $meta_box['name'] ) )
				continue;

			if ( 'checkbox' == $meta_box['type'] ) {
				if ( isset( $_POST['yoast_wpseo_' . $meta_box['name']] ) )
					$data = 'on';
				else
					$data = 'off';
			}
			else if ( 'multiselect' == $meta_box['type'] ) {
				if ( isset( $_POST['yoast_wpseo_' . $meta_box['name']] ) ) {
					if ( is_array( $_POST['yoast_wpseo_' . $meta_box['name']] ) )
						$data = implode( ",", $_POST['yoast_wpseo_' . $meta_box['name']] );
					else
						$data = $_POST['yoast_wpseo_' . $meta_box['name']];
				}
				else {
					continue;
				}
			}
			else {
				if ( isset( $_POST['yoast_wpseo_' . $meta_box['name']] ) )
					$data = $_POST['yoast_wpseo_' . $meta_box['name']];
				else
					continue;
			}

			if ( ! in_array( $data, array( '', '0', 'none', '-', 'index,follow' ) ) ) {
				wpseo_set_value( $meta_box['name'], sanitize_text_field( $data ), $post_id );
			} else if ( $data == '' ) {
				delete_post_meta( $post_id, '_yoast_wpseo_' . $meta_box['name'] );
			}
		}

		do_action( 'wpseo_saved_postdata' );
	}

	public function enqueue() {
		$color = get_user_meta( get_current_user_id(), 'admin_color', true );
		if ( '' == $color || in_array( $color, array( 'classic', 'fresh' ), true ) === false )
			$color = 'fresh';

		global $pagenow;
		if ( $pagenow == 'edit.php' ) {
			wp_enqueue_style( 'edit-page', get_template_directory_uri().'/seo/css/edit-page.css', array(), WPSEO_VERSION );
		}
		else {
			wp_enqueue_style( 'metabox-tabs', get_template_directory_uri().'/seo/css/metabox-tabs.css', array(), WPSEO_VERSION );
			wp_enqueue_style( "metabox-$color", get_template_directory_uri().'/seo/css/metabox-' . esc_attr( $color ) . '.css', array(), WPSEO_VERSION );

			wp_enqueue_script( 'jquery-ui-autocomplete' );

			wp_enqueue_script( 'jquery-qtip', get_template_directory_uri().'/seo/js/jquery.qtip.min.js', array( 'jquery' ), '1.0.0-RC3', true );
			wp_enqueue_script( 'wp-seo-metabox', get_template_directory_uri().'/seo/js/wp-seo-metabox.js', array( 'jquery', 'jquery-ui-core', 'jquery-ui-autocomplete' ), WPSEO_VERSION, true );

		}
	}

	function posts_filter_dropdown() {
		global $pagenow;
		if ( $pagenow == 'upload.php' )
			return false;
	}

	function column_heading( $columns ) {
		return array_merge( $columns, array( 'wpseo-score' => __( 'SEO', 'wordpress-seo' ), 'wpseo-title' => __( 'SEO Title', 'wordpress-seo' ), 'wpseo-metadesc' => __( 'Meta Desc.', 'wordpress-seo' ), 'wpseo-focuskw' => __( 'Focus KW', 'wordpress-seo' ) ) );
	}

	function page_title( $post_id ) {
		$fixed_title = wpseo_get_value( 'title', $post_id );
		if ( $fixed_title ) {
			return $fixed_title;
		}
		else {
			$post    = get_post( $post_id );
			$options = get_wpseo_options();
			if ( isset( $options['title-' . $post->post_type] ) && ! empty( $options['title-' . $post->post_type] ) ) {
				$title_template = $options['title-' . $post->post_type];
				$title_template = str_replace( ' %%page%% ', ' ', $title_template );
				return wpseo_replace_vars( $title_template, (array) $post );
			}
			else {
				return wpseo_replace_vars( '%%title%%', (array) $post );
			}
		}
	}

	function aasort( &$array, $key ) {
		$sorter = array();
		$ret    = array();
		reset( $array );
		foreach ( $array as $ii => $va ) {
			$sorter[$ii] = $va[$key];
		}
		asort( $sorter );
		foreach ( $sorter as $ii => $va ) {
			$ret[$ii] = $array[$ii];
		}
		$array = $ret;
	}


	function strip_separators_and_fold( $inputString, $removeOptionalCharacters = false ) {
		$keywordCharactersAlwaysReplacedBySpace = array( ",", "'", "\"", "?", "’", "“", "”", "|", "/" );
		$keywordCharactersRemovedOrReplaced     = array( "_", "-" );
		$keywordWordsRemoved                    = array( " a ", " in ", " an ", " on ", " for ", " the ", " and " );

		// lower
		$inputString = $this->strtolower_utf8( $inputString );

		// default characters replaced by space
		$inputString = str_replace( $keywordCharactersAlwaysReplacedBySpace, ' ', $inputString );

		// standardise whitespace
		$inputString = preg_replace( '`\s+`u', ' ', $inputString );

		// deal with the separators that can be either removed or replaced by space
		if ( $removeOptionalCharacters ) {
			// remove word separators with a space
			$inputString = str_replace( $keywordWordsRemoved, ' ', $inputString );

			$inputString = str_replace( $keywordCharactersRemovedOrReplaced, '', $inputString );
		}
		else {
			$inputString = str_replace( $keywordCharactersRemovedOrReplaced, ' ', $inputString );
		}

		// standardise whitespace again
		$inputString = preg_replace( '`\s+`u', ' ', $inputString );

		return trim( $inputString );
	}

	function get_body( $post ) {
		// This filter allows plugins to add their content to the content to be analyzed.
		$post_content = apply_filters( 'wpseo_pre_analysis_post_content', $post->post_content, $post );

		// Strip shortcodes, for obvious reasons, if plugins think their content should be in the analysis, they should
		// hook into the above filter.
		$post_content = wpseo_strip_shortcode( $post_content );

		if ( trim( $post_content ) == '' )
			return '';

		$htmdata2 = preg_replace( '`[\n\r]`', ' ', $post_content );
		if ( $htmdata2 == null )
			$htmdata2 = $post_content;
		else
			unset( $post_content );

		$htmdata3 = preg_replace( '`<(?:\x20*script|script).*?(?:/>|/script>)`', '', $htmdata2 );
		if ( $htmdata3 == null )
			$htmdata3 = $htmdata2;
		else
			unset( $htmdata2 );

		$htmdata4 = preg_replace( '`<!--.*?-->`', '', $htmdata3 );
		if ( $htmdata4 == null )
			$htmdata4 = $htmdata3;
		else
			unset( $htmdata3 );

		$htmdata5 = preg_replace( '`<(?:\x20*style|style).*?(?:/>|/style>)`', '', $htmdata4 );
		if ( $htmdata5 == null )
			$htmdata5 = $htmdata4;
		else
			unset( $htmdata4 );

		return $htmdata5;
	}

	function get_first_paragraph( $body ) {
		// To determine the first paragraph we first need to autop the content, then match the first paragraph and return.
		$res = preg_match( '`<p[.]*?>(.*)</p>`', wpautop( strip_tags( $body ) ), $matches );
		if ( $res )
			return $matches[1];
		return false;
	}
}

global $wpseo_metabox;
$wpseo_metabox = new WPSEO_Metabox();