<?php

if ( ! defined( 'WPSEO_VERSION' ) ) {
	header( 'HTTP/1.0 403 Forbidden' );
	die;
}

class WPSEO_OpenGraph extends WPSEO_Frontend {

	var $options = array();

	var $shown_images = array();

	public function __construct() {
		$this->options = get_wpseo_options();

		global $fb_ver;
		if ( isset( $fb_ver ) || class_exists( 'Facebook_Loader' ) ) {
			add_filter( 'fb_meta_tags', array( $this, 'facebook_filter' ), 10, 1 );
		}
		else {
			add_filter( 'language_attributes', array( $this, 'add_opengraph_namespace' ) );
			add_action( 'wpseo_opengraph', array( $this, 'og_title' ), 10 );
			add_action( 'wpseo_opengraph', array( $this, 'site_owner' ), 20 );
			add_action( 'wpseo_opengraph', array( $this, 'description' ), 11 );
			add_action( 'wpseo_opengraph', array( $this, 'url' ), 12 );
			add_action( 'wpseo_opengraph', array( $this, 'article_author_facebook' ), 15 );
			add_action( 'wpseo_opengraph', array( $this, 'image' ), 30 );
		}
		remove_action( 'wp_head', 'jetpack_og_tags' );
		add_action( 'wpseo_head', array( $this, 'opengraph' ), 30 );
	}

	public function opengraph() {
		wp_reset_query();
		do_action( 'wpseo_opengraph' );
	}

	private function og_tag( $property, $content ) {
		$content = apply_filters( 'wpseo_og_' . str_replace( ':', '_', $property ), $content );
		if ( ! empty( $content ) )
			echo '<meta property="' . $property . '" content="' . esc_attr( $content ) . '" />' . "\n";
	}

	public function facebook_filter( $meta_tags ) {
		$meta_tags['http://ogp.me/ns#type']  = $this->type( false );
		$meta_tags['http://ogp.me/ns#title'] = $this->og_title( false );

		$meta_tags['http://ogp.me/ns#locale'] = $this->locale( false );

		$ogdesc = $this->description( false );
		if ( ! empty( $ogdesc ) )
			$meta_tags['http://ogp.me/ns#description'] = $ogdesc;

		return $meta_tags;
	}

	public function add_opengraph_namespace( $input ) {
		return $input . ' prefix="og: http://ogp.me/ns#' . ( ( isset( $this->options['fbadminapp'] ) || isset( $this->options['fb_admins'] ) ) ? ' fb: http://ogp.me/ns/fb#' : '' ) . '"';
	}

	public function article_author_facebook() {
		if ( ! is_singular() )
			return;

		global $post;
		$facebook = apply_filters( 'wpseo_opengraph_author_facebook', get_the_author_meta( 'facebook', $post->post_author ) );

		if ( $facebook && ( is_string( $facebook ) && $facebook !== '' ) )
			$this->og_tag( 'article:author', $facebook );
	}

	public function site_owner() {
		if ( isset( $this->options['fbadminapp'] ) && 0 != $this->options['fbadminapp'] ) {
			$this->og_tag( 'fb:app_id', $this->options['fbadminapp'] );
		}
		else if ( isset( $this->options['fb_admins'] ) && is_array( $this->options['fb_admins'] ) && ( count( $this->options['fb_admins'] ) > 0 ) ) {
			$adminstr = '';
			foreach ( $this->options['fb_admins'] as $admin_id => $admin ) {
				if ( ! empty( $adminstr ) )
					$adminstr .= ',' . $admin_id;
				else
					$adminstr = $admin_id;
			}
			$adminstr = apply_filters( 'wpseo_opengraph_admin', $adminstr );
			if ( is_string( $adminstr ) && $adminstr !== '' )
				$this->og_tag( 'fb:admins', $adminstr );
		}
	}

	public function og_title( $echo = true ) {
		$title = apply_filters( 'wpseo_opengraph_title', $this->title( '' ) );

		if ( is_string( $title ) && $title !== '' ) {
			if ( $echo !== false )
				$this->og_tag( 'og:title', $title );
		}

		return $title;
	}

	public function url() {
		$url = apply_filters( 'wpseo_opengraph_url', $this->canonical( false ) );
		if ( is_string( $url ) && $url !== '' )
			$this->og_tag( 'og:url', esc_url( $url ) );
	}

	function image_output( $img ) {
		if ( empty( $img ) )
			return false;

		$img = trim( apply_filters( 'wpseo_opengraph_image', $img ) );
		if ( ! empty( $img ) ) {
			if ( strpos( $img, 'http' ) !== 0 ) {
				if ( $img[0] != '/' )
					return false;

				$parsed_url = parse_url( home_url() );
				$img        = $parsed_url['scheme'] . '://' . $parsed_url['host'] . $img;
			}

			if ( in_array( $img, $this->shown_images ) )
				return false;

			array_push( $this->shown_images, $img );

			$this->og_tag( 'og:image', esc_url( $img ) );
		}

		return true;
	}

	public function image() {

		global $post;

		if ( is_front_page() ) {
			if ( isset( $this->options['og_frontpage_image'] ) )
				$this->image_output( $this->options['og_frontpage_image'] );
		}

		if ( is_singular() ) {
			$ogimg = wpseo_get_value( 'opengraph-image' );
			if ( $ogimg ) {
				$this->image_output( $ogimg );
				return;
			}

			if ( function_exists( 'has_post_thumbnail' ) && has_post_thumbnail( $post->ID ) ) {
				$thumb = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), apply_filters( 'wpseo_opengraph_image_size', 'original' ) );
				$this->image_output( $thumb[0] );
			}

			$content = apply_filters( 'wpseo_pre_analysis_post_content', $post->post_content, $post );

			if ( preg_match_all( '`<img [^>]+>`', $content, $matches ) ) {
				foreach ( $matches[0] as $img ) {
					if ( preg_match( '`src=(["\'])(.*?)\1`', $img, $match ) )
						$this->image_output( $match[2] );
				}
			}
		}

		if ( count( $this->shown_images ) == 0 && isset( $this->options['og_default_image'] ) )
			$this->image_output( $this->options['og_default_image'] );

	}


	public function description( $echo = true ) {
		$ogdesc = '';

		if ( is_front_page() ) {
			if ( isset( $this->options['og_frontpage_desc'] ) && ! empty ( $this->options['og_frontpage_desc'] ) ) {
				$ogdesc = $this->options['og_frontpage_desc'];
			} else {
				$ogdesc = $this->metadesc( false );
			}
		}

		if ( is_singular() ) {
			$ogdesc = wpseo_get_value( 'opengraph-description' );
			if ( ! $ogdesc )
				$ogdesc = $this->metadesc( false );

			if ( ! $ogdesc ) {
				$ogdesc = str_replace( '[&hellip;]', '&hellip;', strip_tags( get_the_excerpt() ) );
			}
		}

		if ( is_tax() ) {
			$ogdesc = trim( strip_tags( term_description() ) );
		}

		$ogdesc = apply_filters( 'wpseo_opengraph_desc', $ogdesc );

		if ( $ogdesc && ( is_string( $ogdesc ) && $ogdesc != '' ) ) {
			if ( $echo !== false )
				$this->og_tag( 'og:description', $ogdesc );
		}
		return $ogdesc;
	}

}

global $wpseo_og;
$wpseo_og = new WPSEO_OpenGraph;